/*
 * Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/licenses/publicdomain/
 * Contributors: Gary Kwong and Jason Orendorff
 */

function f(y) {
    eval("let (z=2, w=y) { (function () { w.p = 7; })(); }");
}
var x = {};
f(x);
assertEq(x.p, 7);
reportCompare(0, 0, "");
