/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JavaScript Engine testing utilities.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Jesse Ruderman
 *                 Brendan Eich
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//-----------------------------------------------------------------------------
var BUGNUMBER = 346642;
var summary = 'decompilation of destructuring assignment';
var actual = '';
var expect = '';


//-----------------------------------------------------------------------------
test();
//-----------------------------------------------------------------------------

function test()
{
  enterFunc ('test');
  printBugNumber(BUGNUMBER);
  printStatus (summary);

  expect = 'TypeError: NaN is not a constructor';
  actual = 'No Crash';
  try
  {
    try { throw 1; } catch(e1 if 0) { } catch(e2 if (new NaN)) { }
  }
  catch(ex)
  {
    actual = ex + '';
  }
  reportCompare(expect, actual, summary + ': 1');

  expect = /TypeError: x.t (has no properties|is undefined)/;
  actual = 'No Crash';
  try
  {
    z = [1];
    let (x = (undefined ? 3 : z)) { x.t.g }
  }
  catch(ex)
  {
    actual = ex + '';
  }
  reportMatch(expect, actual, summary + ': 2');

  expect = /TypeError: x.t (has no properties|is undefined)/;
  actual = 'No Crash';
  try
  {
    z = [1];
    new Function("let (x = (undefined ? 3 : z)) { x.t.g }")()
      }
  catch(ex)
  {
    actual = ex + '';
  }
  reportMatch(expect, actual, summary + ': 3');

  expect = 'TypeError: b is not a constructor';
  actual = 'No Crash';
  try
  {
    with({x: (new (b = 1))}) (2).x
      }
  catch(ex)
  {
    actual = ex + '';
  }
  reportCompare(expect, actual, summary + ': 4');

  expect = /TypeError: this.zzz (has no properties|is undefined)/;
  actual = 'No Crash';
  try
  {
    (function(){try { } catch(f) { return; } finally { this.zzz.zzz }})();
  }
  catch(ex)
  {
    actual = ex + '';
  }
  reportMatch(expect, actual, summary + ': 5');

  expect = 'TypeError: p.z = <x><y/></x> ? 3 : 4 is not a function';
  actual = 'No Crash';
  try
  {
    (new Function("if(\n({y:5, p: (print).r})) { p={}; (p.z = <x\n><y/></x> ? 3 : 4)(5) }"))();
  }
  catch(ex)
  {
    actual = ex + '';
  }
  reportCompare(expect, actual, summary + ': 6');

  expect = 'TypeError: xx is not a function';
  actual = 'No Crash';
  try
  {
    switch(xx) { case 3: case (new ([3].map)): } const xx;
  }
  catch(ex)
  {
    actual = ex + '';
  }
  reportCompare(expect, actual, summary + ': 7');

  expect = 'TypeError: ++x is not a function';
  actual = 'No Crash';
  try
  {
    let (x=3) ((++x)())
      }
  catch(ex)
  {
    actual = ex + '';
  }
  reportCompare(expect, actual, summary + ': 8');

  expect = 'ReferenceError: x.y is not defined';
  actual = 'No Crash';
  try
  {
    x = {};
    import x.y;
  }
  catch(ex)
  {
    actual = ex + '';
  }
  reportCompare(expect, actual, summary + ': 9');

  exitFunc ('test');
}
