actual = '';
expected = 'foo4,foo4,';

v = 0
let(f = function (y) {
    let(f = function (y) {
        let(f = function (g) {
            for (h in g) {
                if (++y > 3) {
                    appendToActual('foo' + y)
                }
            }
        }) {
            f([y
            for (b in [1, []])]);
            f(['', false])
        }
        v = String
    }) {
        f(y)
        f(y)
    }
}) {
    f(v)
}


assertEq(actual, expected)
