/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JavaScript Engine testing utilities.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Gavin Sharp
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//-----------------------------------------------------------------------------
var BUGNUMBER = 406769;
var summary = 'Regression from bug 398609 caused infinite loop';
var actual = '';
var expect = '';

//-----------------------------------------------------------------------------
test();
//-----------------------------------------------------------------------------

function test()
{
  enterFunc ('test');
  printBugNumber(BUGNUMBER);
  printStatus (summary);

var a0;
var a1;
var a2;
var a3;
var a4;
var a5;
var a6;
var a7;
var a8;
var a9;
var a10;
var a11;
var a12;
var a13;
var a14;
var a15;
var a16;
var a17;
var a18;
var a19;
var a20;
var a21;
var a22;
var a23;
var a24;
var a25;
var a26;
var a27;
var a28;
var a29;
var a30;
var a31;
var a32;
var a33;
var a34;
var a35;
var a36;
var a37;
var a38;
var a39;
var a40;
var a41;
var a42;
var a43;
var a44;
var a45;
var a46;
var a47;
var a48;
var a49;
var a50;
var a51;
var a52;
var a53;
var a54;
var a55;
var a56;
var a57;
var a58;
var a59;
var a60;
var a61;
var a62;
var a63;
var a64;
var a65;
var a66;
var a67;
var a68;
var a69;
var a70;
var a71;
var a72;
var a73;
var a74;
var a75;
var a76;
var a77;
var a78;
var a79;
var a80;
var a81;
var a82;
var a83;
var a84;
var a85;
var a86;
var a87;
var a88;
var a89;
var a90;
var a91;
var a92;
var a93;
var a94;
var a95;
var a96;
var a97;
var a98;
var a99;
var a100;
var a101;
var a102;
var a103;
var a104;
var a105;
var a106;
var a107;
var a108;
var a109;
var a110;
var a111;
var a112;
var a113;
var a114;
var a115;
var a116;
var a117;
var a118;
var a119;
var a120;
var a121;
var a122;
var a123;
var a124;
var a125;
for (var a126 = 1; a126 < ([1,2,3]).length -1; ++a126)
    1;

 
  reportCompare(expect, actual, summary);

  exitFunc ('test');
}
