/* -*- Mode: java; tab-width:8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JavaScript Engine testing utilities.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Nickolay Ponomarev
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


var summary = 'No Syntax Error when trailing space and XML.ignoreWhitespace ' +
    'true';
var BUGNUMBER = 324688;
var actual = 'No Error';
var expect = 'No Error';
START(summary);

function init()
{
    if (typeof Components != 'undefined')
    {
        try
        {
            netscape.security.PrivilegeManager.
                enablePrivilege('UniversalXPConnect');

            var TestObject = {
            observe: function () {
                    try
                    {
                        printBugNumber(BUGNUMBER);
                        printStatus (summary);
                        printStatus('Browser only: requires UniversalXPConnect');

                        printStatus("XML.ignoreWhitespace=" +
                                    XML.ignoreWhitespace);
                        var x = new XML("<a></a> ");
                    }
                    catch(ex2)
                    {
                        actual = ex2 + '';
                    }
                    print('expect = ' + expect);
                    print('actual = ' + actual);
                    TEST(1, expect, actual);
                    END();
                    gDelayTestDriverEnd = false;
                    jsTestDriverEnd();
                }
            };

            var t = Components.classes["@mozilla.org/timer;1"].
                createInstance(Components.interfaces.nsITimer);
            t.init(TestObject, 100, t.TYPE_ONE_SHOT);
        }
        catch(ex)
        {
            printStatus('Requires UniversalXPConnect');
        }
    }
}

if (typeof window != 'undefined')
{
    // delay test driver end
    gDelayTestDriverEnd = true;

    window.addEventListener("load", init, false);
}
else
{
    TEST(1, expect, actual);
    END();
}

