/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JavaScript Engine testing utilities.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//-----------------------------------------------------------------------------
var BUGNUMBER = 363040;
var summary = 'Array.prototype.reduce, Array.prototype.reduceRight';
var actual = '';
var expect = '';


//-----------------------------------------------------------------------------
test();
//-----------------------------------------------------------------------------

function test()
{
  enterFunc ('test');
  printBugNumber(BUGNUMBER);
  printStatus (summary);
 
  function f(x,y) { return '(' + x + '+' + y + ')';}

  var testdesc;
  var arr0elms  = [];
  var arr1elms = [1];
  var arr2elms  = [1, 2];

  testdesc = 'Test reduce of empty array without initializer.';
  try
  {
    expect = 'TypeError: reduce of empty array with no initial value';
    arr0elms.reduce(f);
  }
  catch(ex)
  {
    actual = ex + '';
  }
  reportCompare(expect + '', actual + '', testdesc + ' : ' + expect);

  testdesc = 'Test reduceRight of empty array without initializer.';
  try
  {
    expect = 'TypeError: reduce of empty array with no initial value';
    arr0elms.reduceRight(f);
  }
  catch(ex)
  {
    actual = ex + '';
  }
  reportCompare(expect + '', actual + '', testdesc + ' : ' + expect);

  testdesc = 'Test reduce of empty array with initial value.';
  expect = 'a';
  actual = arr0elms.reduce(f, 'a');
  reportCompare(expect + '', actual + '', testdesc + ' : ' + expect);

  testdesc = 'Test reduceRight of empty array with initial value.';
  expect = 'a';
  actual = arr0elms.reduceRight(f, 'a');
  reportCompare(expect + '', actual + '', testdesc +' : ' + expect);

  testdesc = 'Test reduce of 1 element array with no initializer.';
  expect = '1';
  actual = arr1elms.reduce(f);
  reportCompare(expect + '', actual + '', testdesc + ' : ' + expect);

  testdesc = 'Test reduceRight of 1 element array with no initializer.';
  expect = '1';
  actual = arr1elms.reduceRight(f);
  reportCompare(expect + '', actual + '', testdesc + ' : ' + expect);

  testdesc = 'Test reduce of 2 element array with no initializer.';
  expect = '(1+2)';
  actual = arr2elms.reduce(f);
  reportCompare(expect + '', actual + '', testdesc + ' : ' + expect);

  testdesc = 'Test reduce of 2 element array with initializer.';
  expect = '((a+1)+2)';
  actual = arr2elms.reduce(f,'a');
  reportCompare(expect + '', actual + '', testdesc + ' : ' + expect);

  testdesc = 'Test reduceRight of 2 element array with no initializer.';
  expect = '(2+1)';
  actual = arr2elms.reduceRight(f);
  reportCompare(expect + '', actual + '', testdesc + ' : ' + expect);

  testdesc = 'Test reduceRight of 2 element array with no initializer.';
  expect = '((a+2)+1)';
  actual = arr2elms.reduceRight(f,'a');
  reportCompare(expect + '', actual + '', testdesc + ' : ' + expect);

  exitFunc ('test');
}
